/*
 * YouGrabber
 *
 * A lightweight, ncurses based, multi-threaded command-line YouTube.com
 * video downloader.
 *
 * Copyright (C) 2006-2007 Quetzy Garcia <quetzyg@users.sourceforge.net>
 * http://yougrabber.sourceforge.net/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "yg.h"

int grab(YGdownload *dl)
{
	FILE *stream;
	CURL *handle = NULL;
	CURLcode rc;
	char buf[256];
	char buf2[256];
	char buf3[256];
	char aux[72];
	char *str;
	unsigned dtitle = 0;
	unsigned durl = 0;
	unsigned x = 0;
	unsigned y = 2;

	/* i guess there's a saner way to do this, don't you? */
	int dlprogress(void *clientp, double dltotal, double dlnow,
			double ultotal, double ulnow)
	{
		mvwprintw(dl->window, 0, 17, "(%.1f KB of %.1f KB downloaded)\n",
			dlnow/1024, dltotal/1024);

		update_panels();
		doupdate();

		return 0;
	}

	handle = curl_easy_init();

	mkfilename(dl->file.name, "TEMPORARY", "dat");
	stream = fopen(dl->file.name, "w+");

	mvwprintw(dl->window, y, x+3, "URL: %s\n", dl->connection.url);

	if (!stream) {
		dipclean();

		mvwprintw(dl->window, y+=4, x, "STATUS: %s\n", strerror(errno));

		update_panels();
		doupdate();

		curl_easy_cleanup(handle);

		return 1;
	}
	else {
		/* connection settings */
		curl_easy_setopt(handle, CURLOPT_URL, dl->connection.url);
		curl_easy_setopt(handle, CURLOPT_WRITEDATA, stream);
		curl_easy_setopt(handle, CURLOPT_FOLLOWLOCATION, 1);
		curl_easy_setopt(handle, CURLOPT_NOPROGRESS, 0);
		curl_easy_setopt(handle, CURLOPT_PROGRESSFUNCTION, dlprogress);
		curl_easy_setopt(handle, CURLOPT_PROXY, dl->config->proxy.host);
		curl_easy_setopt(handle, CURLOPT_PROXYPORT, dl->config->proxy.port);
		curl_easy_setopt(handle, CURLOPT_PROXYTYPE, dl->config->proxy.type);
		snprintf(buf, sizeof(buf), "%s:%s", dl->config->proxy.username,
			dl->config->proxy.password);
		curl_easy_setopt(handle, CURLOPT_PROXYUSERPWD, buf);
		curl_easy_setopt(handle, CURLOPT_PROXYAUTH, CURLAUTH_ANYSAFE);

		mvwprintw(dl->window, y+=4, x, "STATUS: downloading data...\n");

		update_panels();
		doupdate();

		rc = curl_easy_perform(handle);

		if (rc) {
			dipclean();

			mvwprintw(dl->window, y++, x, "STATUS: %s\n",
				curl_easy_strerror(rc));

			update_panels();
			doupdate();

			fclose(stream);
			remove(dl->file.name);
			curl_easy_cleanup(handle);
			return 1;
		}

		fflush(stream);
		rewind(stream);

		/* parse video data from HTML */
		while (fgets(buf, sizeof(buf), stream)) {
			if (feof(stream))
				break;

			/* video title */
			if (strstr(buf, dl->config->video.search.title.string)) {
				dtitle = 1;

				strncpy(dl->title, strcut(buf,
					dl->config->video.search.title.delim[0],
					dl->config->video.search.title.delim[1]),
					sizeof(dl->title));
				if (!strncasecmp(dl->title, "YouTube - ", 10))
					memmove(dl->title, dl->title + 10, strlen(dl->title + 9));

				mkfilename(aux, dl->title, "tmp");
			}

			/* video URL fragment */
			if (strstr(buf, dl->config->video.search.url.string)) {
				char *id;
				strcut(buf, dl->config->video.search.url.delim[0], dl->config->video.search.url.delim[1]);
				id = strstr(buf, "video_id:");
				if (!id) goto bad_url_line;
				strcpy(buf2, id);
				strcut(buf2, '\'', '\'');
				if (!*buf2) goto bad_url_line;
				id = strstr(buf, "t:");
				if (!id) goto bad_url_line;
				strcpy(buf3, id);
				strcut(buf3, '\'', '\'');
				if (!*buf3) goto bad_url_line;

				snprintf(dl->connection.url, sizeof(dl->connection.url),
					"%svideo_id=%s&t=%s", dl->config->video.base_url, buf2, buf3);
				durl = 1;
				bad_url_line:;
			}
		}

		/* got all data needed? */
		if (dtitle && durl) {
			rename(dl->file.name, aux);
			rewind(stream);

			curl_easy_setopt(handle, CURLOPT_URL, dl->connection.url);
			rc = curl_easy_perform(handle);

			if (rc) {
				dipclean();

				mvwprintw(dl->window, y, x, "STATUS: %s\n",
					curl_easy_strerror(rc));

				update_panels();
				doupdate();

				fclose(stream);
				remove(aux);
				curl_easy_cleanup(handle);
				return 1;
			}

			mkfilename(dl->file.name, dl->title, "flv");

			fflush(stream);
			rename(aux, dl->file.name);
			fclose(stream);

			/* gather download info */
			curl_easy_getinfo(handle, CURLINFO_EFFECTIVE_URL, &str);
			strncpy(dl->connection.url, str, sizeof(dl->connection.url));
			curl_easy_getinfo(handle, CURLINFO_TOTAL_TIME,
				&dl->connection.time);
			curl_easy_getinfo(handle, CURLINFO_SIZE_DOWNLOAD, &dl->file.size);
			curl_easy_getinfo(handle, CURLINFO_SPEED_DOWNLOAD,
				&dl->connection.speed);

			curl_easy_cleanup(handle);

			dipclean();

			mvwprintw(dl->window, y++, x, "STATUS: download finished!\n");
			mvwprintw(dl->window, y++, x+1, "TITLE: %s\n", dl->title);
			mvwprintw(dl->window, y++, x, "SOURCE: %s\n", dl->connection.url);
			mvwprintw(dl->window, y++, x+2, "FILE: %s\n", dl->file.name);
			mvwprintw(dl->window, y++, x+2,
				"INFO: %.1f KB downloaded in %.1f secs @ %.1f KB/sec\n",
				dl->file.size/1024,	dl->connection.time,
				dl->connection.speed/1024);

			update_panels();
			doupdate();
		}
		/* guess not... :-( */
		else {
			fclose(stream);
			remove(dl->file.name);
			curl_easy_cleanup(handle);

			dipclean();

			mvwprintw(dl->window, y++, x, "STATUS: no video data found\n");
			mvwprintw(dl->window, y++, x+1, "CAUSE: invalid URL\n");
			mvwprintw(dl->window, y++, x+8, "someone screwed up the settings\n");
			mvwprintw(dl->window, y++, x+8, "deprecated %s version\n",
				YG_STRING);
			mvwprintw(dl->window, y++, x+8, "YouTube changed their system\n");
			mvwprintw(dl->window, y++, x+8, "George W. Bush is not dead yet\n");

			update_panels();
			doupdate();

			return 1;
		}
	}

	return 0;
}
