/*
 * YouGrabber
 *
 * A lightweight, ncurses based, multi-threaded command-line YouTube.com
 * video downloader.
 *
 * Copyright (C) 2006-2007 Quetzy Garcia <quetzyg@users.sourceforge.net>
 * http://yougrabber.sourceforge.net/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "yg.h"

#define FILEMAX 1000

unsigned dip;

char *strcut(char *s, char d1, char d2)
{
	char buf[strlen(s)];
	unsigned n;
	unsigned pos;
	unsigned cut;
	unsigned str_ok;

	for (str_ok = pos = cut = 0, n = 1; n < strlen(s); ++n) {
		if (s[n-1] == d1)
			cut=1;

		if (cut) {
			if (s[n] == d2) {
				buf[pos] = 0;
				str_ok = 1;
				break;
			}
			else
				buf[pos++] = s[n];
		}
	}

	if (!str_ok)
		*buf = 0;

	return strcpy(s, buf);
}

char *strtoupper(char *s)
{
	char buf[strlen(s)];
	unsigned n;

	for (n = 0; n < strlen(s); ++n)
		buf[n] = toupper(s[n]);

	buf[n] = 0;

	return strcpy(s, buf);
}

int fexists(const char *fp)
{
	FILE* stream = fopen(fp, "r");

	if (stream) {
		fclose(stream);
		return 1;
	}

	return 0;
}

char *mkfilename(char* dest, char *s, char fext[])
{
	char sufix[5]; /* 4 chars + \0 */
	unsigned pos;
	unsigned i;
	unsigned j;

	for (i = 0; i < FILEMAX; ++i) {
		if (i)
			snprintf(sufix, sizeof(sufix), "_%.3d", i);
		else
			*sufix = 0;

		for (pos = j = 0; j < strlen(s); ++j)
			if (isalpha(s[j]) || isdigit(s[j]))
				dest[pos++] = s[j];
			else if (isspace(s[j]))
				dest[pos++] = '_';

		dest[pos] = 0;

		strncat(dest, sufix, sizeof(dest));
		pos += strlen(sufix);

		dest[pos++] = '.';
		dest[pos] = 0;

		strncat(dest, fext, sizeof(dest));

		if (!fexists(dest))
			return dest;
	}

	return NULL;
}

void dipclean(void)
{
	if (!--dip)
		mvprintw(7, 0, "INFO: nothing else to do [press ESC to exit]\n");
	else
		mvprintw(7, 0, "INFO: %d download(s) remaining...\n", dip);
}
