/*
 * YouGrabber
 *
 * A lightweight, ncurses based, multi-threaded command-line YouTube.com
 * video downloader.
 *
 * Copyright (C) 2006-2007 Quetzy Garcia <quetzyg@users.sourceforge.net>
 * http://yougrabber.sourceforge.net/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "yg.h"

unsigned dip;

int main(int argc, char **argv)
{
	YGconfig *cfg = NULL;
	YGdownload *dl = NULL;
	unsigned parc = argc-1;
	unsigned quit = 0;
	unsigned n;
	dip = parc;

	/* enter ncurses */
	initscr();
	cbreak();
	noecho();
	curs_set(0);
	keypad(stdscr, TRUE);

	printw("%s v%s - Copyright (C) %s <%s>\nfor news and updates visit %s\n\n",
		YG_STRING, YG_VERSION, YG_AUTHOR, YG_EMAIL, YG_URL);

	if (parc < 1) {
		printw("%s: missing URL\nusage: %s [URL]...\n",
			basename(argv[0]), basename(argv[0]));

		refresh();
		endwin();

		return 0;
	}

	cfg = getcfg();

	if (!cfg) {
		printw("not enough memory to allocate configuration settings\n");

		refresh();
		endwin();

		return 1;
	}

	dl = (YGdownload *) malloc((parc)*sizeof(YGdownload));

	if (!dl) {
		printw("not enough memory to allocate %d download structure(s)\n",
			parc);

		refresh();
		endwin();

		return 1;
	}

	/* settings info */
	if (strlen(cfg->path))
		printw("using custom settings from %s\n", cfg->path);
	else
		printw("using default settings\n");

	/* connection info */
	if (strlen(cfg->proxy.host))
		printw("connection method: proxy [%s:%ld|%ld]\n", cfg->proxy.host,
			cfg->proxy.port, cfg->proxy.type);
	else
		printw("connection method: direct\n");

	/* usage tip */
	mvprintw(LINES-1, 0,
		"TIP: use LEFT and RIGHT arrow keys to browse between downloads\n");

	refresh();

	for(n = 0; n < parc; ++n) {
		strncpy(dl[n].connection.url, argv[n+1], sizeof(dl[n].connection.url));
		dl[n].config = cfg;

		dl[n].window = newwin(13, 0, 10, 0);
		mvwprintw(dl[n].window, 0, 0, "DOWNLOAD %.3d/%.3d\n", n+1, parc);
		mvwhline(dl[n].window, 1, 0, ACS_HLINE, COLS);
		mvwhline(dl[n].window, 12, 0, ACS_HLINE, COLS);
		dl[n].panel = new_panel(dl[n].window);

		if (pthread_create(&dl[n].tid, NULL, (void *) grab, &dl[n])) {
			dipclean();

			mvwprintw(dl[n].window, 6, 0,
				"STATUS: unable to run thread for %s\n", dl[n].connection.url);
		}
	}

	/* show first panel */
	n = 0;
	top_panel(dl[n].panel);

	/* take a spin */
	while (!quit) {
		switch(getch()) {
			case KEY_LEFT:
				n = (n+parc-1)%parc;
				break;

			case KEY_RIGHT:
				n = (n+1)%parc;
				break;

			case 27: /* ESC */
				if (!dip)
					quit = 1;
				else
					mvprintw(7, 0, "INFO: cannot exit yet, "
						"%d download(s) in progress!\n", dip);
				break;
		}

		top_panel(dl[n].panel);
		update_panels();
		doupdate();
	}

	clear();
	refresh();

	/* leave ncurses */
	endwin();

	free(dl);
	free(cfg);

	return 0;
}
